try:
    import MySQLdb
except ImportError:
    import sys
    MySQLdb = None
    print "Error: MySQL-python module cannot be loaded"
    sys.exit(1)


class MySQLConnector:
    socket_file = None
    host = None
    port = None
    user = None
    passwd = None
    db = None
    query = None
    connection = None

    def __init__(self, socket_file=None, host=None, port=None, user="", passwd="", db=""):
        self.socket_file = socket_file
        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.db = db

    def get(self, query):
        return self._read(query)

    def _connect(self):
        if self.connection is not None:
            return

        if self.socket_file is not None:
            self.connection = MySQLdb.connect(unix_socket=self.socket_file, user=self.user, passwd=self.passwd,
                                              db=self.db)
        else:
            self.connection = MySQLdb.connect(host=self.host, port=self.port, user=self.user, passwd=self.passwd,
                                              db=self.db)

        self.connection.autocommit(True)

    def _read(self, query):
        self._connect()
        cursor = self.connection.cursor(MySQLdb.cursors.DictCursor)

        cursor.execute(query)
        items = cursor.fetchall()
        cursor.close()

        return items



#!/usr/bin/env python
"""
    ZTCCheck class - wrapper for ztc checks
    provides logging, output formatting and error handling abilities

    This file is part of ZTC and distributed under same license.

    Copyright (c) 2011 Denis Seleznyov [https://bitbucket.org/xy2/]
    Copyright (c) 2011 Vladimir Rusinov <vladimir@greenmice.info>
    Copyright (c) 2011 Murano Software [http://muranosoft.com]

    Modified for use in Zabbix EMS by Marijn Giesen <marijn@studio-donder.nl>
"""
import json
import os
import sys
import logging
import logging.handlers
import traceback
import ConfigParser
from enum import Enum
import re


class Check(object):
    name = "zems"
    config = None
    debug = False
    logger = None
    confdir = "/etc/zems"

    metrics = None
    test_data = None
    test_data_filtered = None

    def __init__(self, name=None):
        if name is not None:
            self.name = name
        if self.name == 'zems':
            raise NotImplementedError("Class %s must redefine its name" % (self.__class__.__name__, ))

        self.config = self._get_config()

        if self.config.get('debug', False):
            self.debug = True

        self.logger = logging.getLogger(self.__class__.__name__)
        self._set_logger_options()

        self.logger.debug("config file: %s" % os.path.join(self.confdir, self.name + ".conf"))

        self._init_metrics()

    def _init_metrics(self):
        raise NotImplementedError("Class %s must reimplement _init_metrics method"
                                  % (self.__class__.__name__, ))

    def _get(self, *args, **kwargs):
        raise NotImplementedError("Class %s must reimplement _get method"
                                  % (self.__class__.__name__, ))

    def get(self, metric=None, *args, **kwargs):
        self.logger.debug("executed get metric '%s', args '%s', kwargs '%s'" %
                          (str(metric), str(args), str(kwargs)))
        try:
            metric = metric.lower()
            if metric not in self.metrics:
                raise CheckFail("Requested unknown metric: %s" % metric)

            print self._get(metric, *args, **kwargs)
        except CheckFail, e:
            self.logger.exception('Check fail, getting %s' % (metric, ))
            if self.debug:
                traceback.print_stack()
            for arg in e.args:
                print(arg)
            sys.exit(1)
        except CheckTimeout, e:
            self.logger.exception('Check timeout, getting %s' % (metric, ))
            if self.debug:
                traceback.print_stack()
            for arg in e.args:
                print(arg)
            sys.exit(2)
        except Exception, e:
            self.logger.exception('Check unexpected error, getting %s' % (metric, ))
            sys.exit(1)

    def print_metrics(self):
        print "Available metrics:"
        for metric in sorted(self.metrics.keys()):
            print "  %s" % metric

    def need_root(self):
        if os.getuid() != 0:
            self.logger.exception("Need root privileges to perform this check")
            sys.exit(1)

    def _correct_type(self, type, value):
        value = str(value)
        if type == MetricType.String:
            return value.strip()
        elif type == MetricType.Float:
            if len(value) == 0:
                return 0.0
            return float(value)
        elif type == MetricType.Integer:
            if len(value) == 0:
                return 0
            return int(value)
        elif type == MetricType.Discovery:
            return self._to_discovery_output(value)
        else:
            raise CheckFail("Unknown return type")

    def _get_config(self):
        config = MyConfigParser()
        config.read(os.path.join(self.confdir, self.name + ".conf"))
        return config

    def _set_logger_options(self):
        formatter = logging.Formatter("[%(name)s] %(asctime)s - %(levelname)s: %(message)s")

        # setting file handler (max 10 files of 1MB)
        h = logging.handlers.RotatingFileHandler(self.config.get("logfile", "/var/log/zems.log"), "a", 1 * 1024 * 1024, 10)

        if self.debug:
            # setting stream handler
            sh = logging.StreamHandler()
            sh.setLevel(logging.DEBUG)
            self.logger.setLevel(logging.DEBUG)
            sh.setFormatter(formatter)
            h.setLevel(logging.DEBUG)
            self.logger.addHandler(sh)
        else:
            self.logger.setLevel(logging.WARN)
            h.setLevel(logging.WARN)

        h.setFormatter(formatter)
        self.logger.addHandler(h)
        self.logger.debug("created")

    def _to_discovery_output(self, data):
        # {
        # "data":[
        #
        # { "{#FSNAME}":"\/",                           "{#FSTYPE}":"rootfs"   },
        # { "{#FSNAME}":"\/sys",                        "{#FSTYPE}":"sysfs"    },
        # { "{#FSNAME}":"\/proc",                       "{#FSTYPE}":"proc"     },
        # { "{#FSNAME}":"\/dev",                        "{#FSTYPE}":"devtmpfs" },
        #   { "{#FSNAME}":"\/dev\/pts",                   "{#FSTYPE}":"devpts"   },
        #   { "{#FSNAME}":"\/",                           "{#FSTYPE}":"ext3"     },
        #   { "{#FSNAME}":"\/lib\/init\/rw",              "{#FSTYPE}":"tmpfs"    },
        #   { "{#FSNAME}":"\/dev\/shm",                   "{#FSTYPE}":"tmpfs"    },
        #   { "{#FSNAME}":"\/home",                       "{#FSTYPE}":"ext3"     },
        #   { "{#FSNAME}":"\/tmp",                        "{#FSTYPE}":"ext3"     },
        #   { "{#FSNAME}":"\/usr",                        "{#FSTYPE}":"ext3"     },
        #   { "{#FSNAME}":"\/var",                        "{#FSTYPE}":"ext3"     },
        #   { "{#FSNAME}":"\/sys\/fs\/fuse\/connections", "{#FSTYPE}":"fusectl"  }
        #
        #   ]
        # }

        return json.dumps({"data": data})


class MetricType(Enum):
    Integer = 0
    Float = 1
    String = 2
    Discovery = 3


class Metric(object):
    key = None
    position = None
    type = None
    separator = None
    filter_callback = None
    kwargs = {}

    def __init__(self, key, type, position, separator=None, filter_callback=None, regex=None, **kwargs):
        self.key = key
        self.position = position
        self.type = type
        self.separator = separator
        self.filter_callback = filter_callback
        self.kwargs = kwargs

        if regex is not None:
            self.regex = re.compile(regex)

    def __repr__(self):
        return self.key


class MyConfigParser(ConfigParser.ConfigParser):
    def __init__(self, section='main'):
        self.sectname = section
        ConfigParser.ConfigParser.__init__(self)

    def get(self, option, default):
        try:
            return ConfigParser.ConfigParser.get(self, self.sectname, option)
        except Exception, e:
            return default


class CheckFail(Exception):
    pass


class CheckTimeout(Exception):
    pass


import os
from time import time


class Cache:
    def __init__(self):
        pass

    @staticmethod
    def write(name, data):
        with open("/tmp/zems-%s" % name, "w") as tmpfile:
            tmpfile.writelines(data)

    @staticmethod
    def read(name, ttl=59):
        if Cache.is_expired(name, ttl):
            return None

        try:
            with open("/tmp/zems-%s" % name, "r") as tmpfile:
                return tmpfile.readlines()
        except OSError:
            return None

    @staticmethod
    def is_expired(name, ttl):
        try:
            if os.path.getmtime("/tmp/zems-%s" % name) <= (time() - ttl):
                return True
        except OSError:
            return True

        return False


import os

from check import Check, CheckFail, MetricType, Metric
from lib import utils
from lib.cache import Cache


class RdiffBackup(Check):
    name = "rdiff-backup"

    def _init_metrics(self):
        self.metrics = {
            "start_time": Metric("StartTime", MetricType.Float, 0, " "),
            "end_time": Metric("EndTime", MetricType.Float, 1, " "),
            "elapsed_time": Metric("ElapsedTime", MetricType.Float, 2, " "),
            "source_files": Metric("SourceFiles", MetricType.Integer, 3, " "),
            "source_file_size": Metric("SourceFileSize", MetricType.Integer, 4, " "),
            "mirror_files": Metric("MirrorFiles", MetricType.Integer, 5, " "),
            "mirror_file_size": Metric("MirrorFileSize", MetricType.Integer, 6, " "),
            "new_files": Metric("NewFiles", MetricType.Integer, 7, " "),
            "new_file_size": Metric("NewFileSize", MetricType.Integer, 8, " "),
            "deleted_files": Metric("DeletedFiles", MetricType.Integer, 9, " "),
            "deleted_file_size": Metric("DeletedFileSize", MetricType.Integer, 10, " "),
            "changed_files": Metric("ChangedFiles", MetricType.Integer, 11, " "),
            "changed_source_size": Metric("ChangedSourceSize", MetricType.Integer, 12, " "),
            "changed_mirror_size": Metric("ChangedMirrorSize", MetricType.Integer, 13, " "),
            "increment_files": Metric("IncrementFiles", MetricType.Integer, 14, " "),
            "increment_file_size": Metric("IncrementFileSize", MetricType.Integer, 15, " "),
            "total_dest_size_change": Metric("TotalDestinationSizeChange", MetricType.Integer, 16, " "),
            "errors": Metric("Errors", MetricType.Integer, 17, " "),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name, 3000)
        if self.test_data is not None:
            return self.test_data

        with open(self._get_statistics_file(), "r") as f:
            self.test_data = f.readlines()

        Cache.write(self.name, self.test_data)

        return self.test_data

    def _get_statistics_file(self):
        files = utils.find_files(
            os.path.join(self.config.get("rdiff_backup_path", "/mnt/backup/system"), "rdiff-backup-data"),
            "session_statistics")
        return utils.determine_newest_file(files)





import os


def find_files_by_extension(path, extension):
    if not extension.startswith("."):
        extension = "." + extension

    files = [
        os.path.join(root, filename)
        for root, dirs, files in os.walk(path)
        for filename in files if filename.endswith(extension)
    ]

    if len(files) < 1:
        raise IOError("No files found with extension %s" % extension)

    return files


def find_files(path, name):
    files = [
        os.path.join(root, filename)
        for root, dirs, files in os.walk(path)
        for filename in files if name in filename
    ]

    if len(files) < 1:
        raise IOError("No files found with name %s" % name)

    return files


def determine_newest_file(files):
    modified_at = 0
    newest_file = None

    for filename in files:
        tmp = os.path.getmtime(filename)
        if tmp > modified_at:
            modified_at = tmp
            newest_file = filename

    return newest_file


from check import Check, CheckFail, MetricType, Metric
from lib.urlconnector import UrlConnector
from lib.cache import Cache


class Apache(Check):
    name = "apache"

    def _init_metrics(self):
        self.metrics = {
            "workers_busy": Metric("BusyWorkers", MetricType.Integer, 0, ":"),
            "workers_idle": Metric("IdleWorkers", MetricType.Integer, 1, ":"),
            "workers_closing": Metric("C", MetricType.Integer, 2, ":", self._filter_data),
            "workers_dns": Metric("D", MetricType.Integer, 2, ":", self._filter_data),
            "workers_finishing": Metric("G", MetricType.Integer, 2, ":", self._filter_data),
            "workers_idlecleanup": Metric("I", MetricType.Integer, 2, ":", self._filter_data),
            "workers_keepalive": Metric("K", MetricType.Integer, 2, ":", self._filter_data),
            "workers_logging": Metric("L", MetricType.Integer, 2, ":", self._filter_data),
            "workers_openslot": Metric(".", MetricType.Integer, 2, ":", self._filter_data),
            "workers_reading": Metric("R", MetricType.Integer, 2, ":", self._filter_data),
            "workers_starting": Metric("S", MetricType.Integer, 2, ":", self._filter_data),
            "workers_writing": Metric("W", MetricType.Integer, 2, ":", self._filter_data),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        if metric.filter_callback is not None:
            metric.filter_callback(metric.position, metric.separator, metric.key)

        return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self.test_data

        url = "%s://%s:%s%s?auto" % (
            self.config.get("proto", "http"), self.config.get("host", "localhost"), self.config.get("port", "80"),
            self.config.get("resource", "/server-status")
        )

        connector = UrlConnector(url)
        data = connector.get()
        if data.status_code != 200:
            raise CheckFail("Unable to retrieve data (error code: %s)" % data.status_code)

        data = data.text.strip()
        Cache.write(self.name, data)

        return data.split("\n")

    def _filter_data(self, linenumber, separator, key):
        data = self.test_data[linenumber].split(separator)[1].count(key)
        self.test_data[linenumber] = str(key) + ":" + str(data)



import requests


class UrlConnector:
    url = None

    def __init__(self, url):
        self.url = url

    def get(self):
        return self._read()

    def _read(self):
        response = requests.get(self.url)
        return response


from lib.socketconnector import *
from check import Check, CheckFail, MetricType, Metric
from lib.cache import Cache


class HAProxy(Check):
    name = "haproxy"
    pxname = None
    svname = None

    def _init_metrics(self):
        self.metrics = {
            "version": Metric("version", MetricType.String, 1, ":"),
            "uptime_sec": Metric("uptime_sec", MetricType.Integer, 7, ":"),
            "memmax_mb": Metric("memmax_mb", MetricType.Integer, 8, ":"),
            "maxsock": Metric("maxsock", MetricType.Integer, 10, ":"),
            "maxconn": Metric("maxconn", MetricType.Integer, 11, ":"),
            "maxpipes": Metric("maxpipes", MetricType.Integer, 12, ":"),
            "currconns": Metric("currconns", MetricType.Integer, 13, ":"),
            "pipesused": Metric("pipesused", MetricType.Integer, 14, ":"),
            "pipesfree": Metric("pipesfree", MetricType.Integer, 15, ":"),
            "tasks": Metric("tasks", MetricType.Integer, 16, ":"),
            "run_queue": Metric("run_queue", MetricType.Integer, 17, ":"),
            "node": Metric("node", MetricType.String, 18, ":"),
            "qcur": Metric("qcur", MetricType.Integer, 2, ",", self._filter_data),
            "qmax": Metric("qmax", MetricType.Integer, 3, ",", self._filter_data),
            "scur": Metric("scur", MetricType.Integer, 4, ",", self._filter_data),
            "smax": Metric("smax", MetricType.Integer, 5, ",", self._filter_data),
            "slim": Metric("slim", MetricType.Integer, 6, ",", self._filter_data),
            "stot": Metric("stot", MetricType.Integer, 7, ",", self._filter_data),
            "bin": Metric("bin", MetricType.Integer, 8, ",", self._filter_data),
            "bout": Metric("bout", MetricType.Integer, 9, ",", self._filter_data),
            "dreq": Metric("dreq", MetricType.Integer, 10, ",", self._filter_data),
            "dresp": Metric("dresp", MetricType.Integer, 11, ",", self._filter_data),
            "ereq": Metric("ereq", MetricType.Integer, 12, ",", self._filter_data),
            "econ": Metric("econ", MetricType.Integer, 13, ",", self._filter_data),
            "eresp": Metric("eresp", MetricType.Integer, 14, ",", self._filter_data),
            "wretr": Metric("wretr", MetricType.Integer, 15, ",", self._filter_data),
            "wredis": Metric("wredis", MetricType.Integer, 16, ",", self._filter_data),
            "status": Metric("status", MetricType.String, 17, ",", self._filter_data),
            "weight": Metric("weight", MetricType.Integer, 18, ",", self._filter_data),
            "act": Metric("act", MetricType.String, 19, ",", self._filter_data),
            "bck": Metric("bck", MetricType.String, 20, ",", self._filter_data),
            "chkfail": Metric("chkfail", MetricType.String, 21, ",", self._filter_data),
            "chkdown": Metric("chkdown", MetricType.String, 22, ",", self._filter_data),
            "lastchg": Metric("lastchg", MetricType.String, 23, ",", self._filter_data),
            "downtime": Metric("downtime", MetricType.String, 24, ",", self._filter_data),
            "qlimit": Metric("qlimit", MetricType.String, 25, ",", self._filter_data),
            "pid": Metric("pid", MetricType.String, 26, ",", self._filter_data),
            "iid": Metric("iid", MetricType.String, 27, ",", self._filter_data),
            "sid": Metric("sid", MetricType.String, 28, ",", self._filter_data),
            "throttle": Metric("throttle", MetricType.String, 29, ",", self._filter_data),
            "lbtot": Metric("lbtot", MetricType.Integer, 30, ",", self._filter_data),
            "tracked": Metric("tracked", MetricType.String, 31, ",", self._filter_data),
            "type": Metric("type", MetricType.String, 32, ",", self._filter_data),
            "rate": Metric("rate", MetricType.String, 33, ",", self._filter_data),
            "rate_lim": Metric("rate_lim", MetricType.String, 34, ",", self._filter_data),
            "rate_max": Metric("rate_max", MetricType.String, 35, ",", self._filter_data),
            "check_status": Metric("check_status", MetricType.String, 36, ",", self._filter_data),
            "check_code": Metric("check_code", MetricType.String, 37, ",", self._filter_data),
            "check_duration": Metric("check_duration", MetricType.String, 38, ",", self._filter_data),
            "hrsp_1xx": Metric("hrsp_1xx", MetricType.String, 39, ",", self._filter_data),
            "hrsp_2xx": Metric("hrsp_2xx", MetricType.String, 40, ",", self._filter_data),
            "hrsp_3xx": Metric("hrsp_3xx", MetricType.String, 41, ",", self._filter_data),
            "hrsp_4xx": Metric("hrsp_4xx", MetricType.String, 42, ",", self._filter_data),
            "hrsp_5xx": Metric("hrsp_5xx", MetricType.String, 43, ",", self._filter_data),
            "hrsp_other": Metric("hrsp_other", MetricType.String, 44, ",", self._filter_data),
            "hanafail": Metric("hanafail", MetricType.String, 45, ",", self._filter_data),
            "req_rate": Metric("req_rate", MetricType.String, 46, ",", self._filter_data),
            "req_rate_max": Metric("req_rate_max", MetricType.String, 47, ",", self._filter_data),
            "req_tot": Metric("req_tot", MetricType.String, 48, ",", self._filter_data),
            "cli_abrt": Metric("cli_abrt", MetricType.String, 49, ",", self._filter_data),
            "srv_abrt": Metric("srv_abrt", MetricType.String, 50, ",", self._filter_data),
            "discovery": Metric("discovery", MetricType.Discovery, 1, ",", self._discovery),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.pxname = kwargs.get("pxname", None)
        self.svname = kwargs.get("svname", None)

        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        if metric.filter_callback is not None:
            if metric.type == MetricType.Discovery:
                metric.filter_callback()
                return self._correct_type(metric.type, self.test_data)
            else:
                metric.filter_callback(metric.separator)
                return self._correct_type(metric.type, self.test_data[metric.position])
        else:
            return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self.test_data

        connector = SocketConnector(socket_file=self.config.get("socket", "/var/lib/haproxy/stats.sock"),
                                    command="show info\nshow stat\n")

        data = connector.get()
        Cache.write(self.name, data)

        return data.split("\n")

    def _filter_data(self, separator):
        if self.pxname is None or self.svname is None:
            raise CheckFail("Required parameters not set (pxname, svname)")

        for id, line in enumerate(self.test_data):
            values = line.split(separator)
            if self.pxname in values and self.svname in values:
                self.test_data = values

    def _discovery(self):
        data = [value.strip().split(",", 2)[0:2]
                for nr, value in enumerate(self.test_data[22:])
                if "#" not in value and value.strip() != ""]

        self.test_data = [{"{#PROXY}": item[0], "{#SERVER}": item[1]}
                          for item in data]



import flup_fcgi_client as fcgi_client


class FcgiConnector:
    socket_file = None
    host = None
    port = None
    uri = None

    def __init__(self, socket_file=None, host=None, port=None,
                 uri=None):
        self.socket_file = socket_file
        self.host = host
        self.port = int(port)
        self.uri = uri

    def get(self):
        return self._read()

    def _read(self):
        if self.socket_file is not None:
            fcgi = fcgi_client.FCGIApp(connect=self.socket_file)
        elif self.host is not None and self.port is not None:
            fcgi = fcgi_client.FCGIApp(host=self.host, port=self.port)
        else:
            raise Exception("No socket_file, host or port given as parameter")

        if self.uri is None:
            raise Exception("No valid URI given")

        env = {
            'SCRIPT_FILENAME': self.uri,
            'QUERY_STRING': '',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': self.uri,
            'REQUEST_URI': self.uri,
            'GATEWAY_INTERFACE': 'CGI/1.1',
            'SERVER_SOFTWARE': 'zems',
            'REDIRECT_STATUS': '200',
            'CONTENT_TYPE': '',
            'CONTENT_LENGTH': '0',
            # 'DOCUMENT_URI': url,
            'DOCUMENT_ROOT': '/var/www/'
        }

        return fcgi(env)



from lib.socketconnector import *
from check import Check, CheckFail, MetricType, Metric
from zems.lib.cache import Cache


class Redis(Check):
    name = "redis"
    db = None

    def _init_metrics(self):
        self.metrics = {
            "redis_version": Metric("redis_version", MetricType.String, 2, ":"),
            "uptime_in_seconds": Metric("uptime_in_seconds", MetricType.Integer, 14, ":"),
            "connected_clients": Metric("connected_clients", MetricType.Integer, 21, ":"),
            "client_longest_output_list": Metric("client_longest_output_list", MetricType.Integer, 22, ":"),
            "client_biggest_input_buf": Metric("client_biggest_input_buf", MetricType.Integer, 23, ":"),
            "blocked_clients": Metric("blocked_clients", MetricType.Integer, 24, ":"),
            "used_memory": Metric("used_memory", MetricType.Integer, 27, ":"),
            "used_memory_rss": Metric("used_memory_rss", MetricType.Integer, 29, ":"),
            "used_memory_peak": Metric("used_memory_peak", MetricType.Integer, 30, ":"),
            "used_memory_lua": Metric("used_memory_lua", MetricType.Integer, 32, ":"),
            "mem_fragmentation_ratio": Metric("mem_fragmentation_ratio", MetricType.Float, 33, ":"),
            "rdb_changes_since_last_save": Metric("rdb_changes_since_last_save", MetricType.Integer, 38, ":"),
            "rdb_last_save_time": Metric("rdb_last_save_time", MetricType.Integer, 40, ":"),
            "rdb_last_bgsave_status": Metric("rdb_last_bgsave_status", MetricType.String, 41, ":"),
            "rdb_last_bgsave_time_sec": Metric("rdb_last_bgsave_time_sec", MetricType.Integer, 42, ":"),
            "total_connections_received": Metric("total_connections_received", MetricType.Integer, 53, ":"),
            "total_commands_processed": Metric("total_commands_processed", MetricType.Integer, 54, ":"),
            "rejected_connections": Metric("rejected_connections", MetricType.Integer, 56, ":"),
            "expired_keys": Metric("expired_keys", MetricType.Integer, 60, ":"),
            "evicted_keys": Metric("evicted_keys", MetricType.Integer, 61, ":"),
            "keyspace_hits": Metric("keyspace_hits", MetricType.Integer, 62, ":"),
            "keyspace_misses": Metric("keyspace_misses", MetricType.Integer, 63, ":"),
            "connected_slaves": Metric("connected_slaves", MetricType.Integer, 70, ":"),
            "master_repl_offset": Metric("master_repl_offset", MetricType.Integer, 71, ":"),
            "used_cpu_sys": Metric("used_cpu_sys", MetricType.Float, 78, ":"),
            "used_cpu_user": Metric("used_cpu_user", MetricType.Float, 79, ":"),
            "used_cpu_sys_children": Metric("used_cpu_sys_children", MetricType.Float, 80, ":"),
            "used_cpu_user_children": Metric("used_cpu_user_children", MetricType.Float, 81, ":"),
            "keys": Metric("keys", MetricType.Integer, 0, "=", self._filter_data),
            "expires": Metric("expires", MetricType.Integer, 1, "=", self._filter_data),
            "avg_ttl": Metric("avg_ttl", MetricType.Integer, 2, "=", self._filter_data),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.db = kwargs.get("db", None)

        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        if metric.filter_callback is not None:
            metric.filter_callback(":")

        return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self.test_data

        connector = SocketConnector(host=self.config.get("host", "localhost"),
                                    port=self.config.get("port", "6379"),
                                    command="info\r\nquit\r\n")

        data = connector.get().strip("\r")
        Cache.write(self.name, data)

        return data.split("\n")

    def _filter_data(self, separator):
        if self.db is None:
            raise CheckFail("Required parameters not set (db)")

        for db_id, line in enumerate(self.test_data):
            values = line.split(separator)

            if self.db in values:
                self.test_data = values[1].split(",")




from check import Check, CheckFail, MetricType, Metric
from lib.urlconnector import UrlConnector
from lib.cache import Cache


class Nginx(Check):
    name = "nginx"

    def _init_metrics(self):
        self.metrics = {
            "connections_active": Metric("active connections", MetricType.Integer, 0, ":"),
            "connections_reading": Metric("reading", MetricType.Integer, 1, " ", self._filter_data, linenumber=1),
            "connections_writing": Metric("writing", MetricType.Integer, 3, " ", self._filter_data, linenumber=1),
            "connections_waiting": Metric("waiting", MetricType.Integer, 5, " ", self._filter_data, linenumber=1),
            "accepts": Metric("accepts", MetricType.Integer, 0, " ", self._filter_data, linenumber=2),
            "handled": Metric("handled", MetricType.Integer, 1, " ", self._filter_data, linenumber=2),
            "requests": Metric("requests", MetricType.Integer, 2, " ", self._filter_data, linenumber=2),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        if metric.filter_callback is not None:
            metric.filter_callback(metric.kwargs.get("linenumber"), metric.separator)
            return self._correct_type(metric.type, self.test_data[metric.position])

        return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self.test_data

        url = "%s://%s:%s%s" % (
            self.config.get("proto", "http"), self.config.get("host", "localhost"), self.config.get("port", "80"),
            self.config.get("resource", "/nginx-status")
        )

        connector = UrlConnector(url)
        data = connector.get()
        if data.status_code != 200:
            raise CheckFail("Unable to retrieve data (error code: %s)" % data.status_code)

        data = data.text.strip()
        Cache.write(self.name, data)

        return data.split("\n")

    def _filter_data(self, linenumber, separator):
        self.test_data = self.test_data[linenumber].strip().split(separator)





from socket import *


class SocketConnector:
    family = None
    type = None
    socket_file = None
    host = None
    port = None
    command = None
    socket = None

    def __init__(self, socket_file=None, host=None, port=None,
                 command=None):
        if socket_file is not None:
            self.socket_file = socket_file
            self.family = AF_UNIX
        else:
            self.host = host
            self.port = int(port)
            self.family = AF_INET
        self.type = SOCK_STREAM
        self.command = command
        self.socket = socket(self.family, self.type)

    def get(self, command=None):
        self._connect()
        self._write(command)

        return self._read()

    def _connect(self):
        if self.socket_file is not None:
            self.socket.connect(self.socket_file)
        elif self.host is not None and self.port is not None:
            self.socket.connect((self.host, self.port))

    def _read(self):
        buf = ""
        while True:
            data = self.socket.recv(1024)
            buf += data
            if not data:
                self.socket.close()
                return buf

    def _write(self, command=None):
        if command is not None:
            self.command = command

        self.socket.send(self.command)



from commandr import command


@command("apache")
def apache(metric=None):
    from zems.apache import Apache

    run_test(Apache(), metric)


@command("haproxy")
def haproxy(metric=None, pxname=None, svname=None):
    from zems.haproxy import HAProxy

    run_test(HAProxy(), metric, need_root=True, pxname=pxname, svname=svname)


@command("nginx")
def nginx(metric=None):
    from zems.nginx import Nginx

    run_test(Nginx(), metric)


@command("php-fpm")
def phpfpm(metric=None, pool=None):
    from zems.phpfpm import PhpFpm

    run_test(PhpFpm(), metric, pool=pool)


@command("rdiff-backup")
def rdiffbackup(metric=None):
    from zems.rdiffbackup import RdiffBackup

    run_test(RdiffBackup(), metric, need_root=True)


@command("redis")
def redis(metric=None, db=None):
    from zems.redis import Redis

    run_test(Redis(), metric, db=db)


@command("sphinx")
def sphinx(metric=None):
    from zems.sphinx import Sphinx

    run_test(Sphinx(), metric)


def run_test(test, metric, need_root=False, **kwargs):
    if metric is not None:
        if need_root:
            test.need_root()
        test.get(metric, **kwargs)
    else:
        test.print_metrics()



import glob
from setuptools import setup

setup(
    name='zems',
    version='0.0.7',
    packages=['zems', 'zems.lib'],
    url='https://github.com/marijngiesen/zabbix-ems',
    license='GPL',
    author='Marijn Giesen',
    author_email='marijn@studio-donder.nl',
    description='ZEMS (Zabbix Extended Monitoring Scripts) is a tool to retrieve all sorts of metrics from '
                'applications and deliver it to Zabbix in a generic way.',
    entry_points={
        'console_scripts': [
            'zems = zems.main:main',
        ]
    },
    install_requires=['requests', 'enum34', 'commandr'],
    data_files=[
        ('/etc/zems/zabbix/', glob.glob('config/zabbix/*.conf')),
        ('/etc/zems', glob.glob('config/*.conf')),
        ('/usr/share/zems/templates', glob.glob('templates/*.xml')),
        ('/usr/share/zems/doc/', ('README.md', 'LICENSE')),
    ]
)



from ConfigParser import ConfigParser

from check import Check, CheckFail, MetricType, Metric
from lib.fcgiconnector import FcgiConnector
from lib import utils
from zems.lib.cache import Cache


class PhpFpm(Check):
    name = "php-fpm"
    pool = None
    fpm_config = None

    def _init_metrics(self):
        self.metrics = {
            "pool": Metric("pool", MetricType.String, 0, ":"),
            "process_manager": Metric("process manager", MetricType.String, 1, ":"),
            "start_since": Metric("start since", MetricType.Integer, 3, ":"),
            "accepted_conn": Metric("accepted conn", MetricType.String, 4, ":"),
            "listen_queue": Metric("listen queue", MetricType.Integer, 5, ":"),
            "listen_queue_max": Metric("max listen queue", MetricType.Integer, 6, ":"),
            "listen_queue_len": Metric("listen queue len", MetricType.Integer, 7, ":"),
            "idle_processes": Metric("idle processes", MetricType.Integer, 8, ":"),
            "active_processes": Metric("active processes", MetricType.Integer, 9, ":"),
            "total_processes": Metric("total processes", MetricType.Integer, 10, ":"),
            "active_processes_max": Metric("max active processes", MetricType.Integer, 11, ":"),
            "max_children_reached": Metric("max children reached", MetricType.Integer, 12, ":"),
            "slow_requests": Metric("slow requests", MetricType.Integer, 13, ":"),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.pool = kwargs.get("pool", None)
        if self.pool is None:
            raise CheckFail("Required parameters not set (pool)")

        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        return self._correct_type(metric.type, self.test_data[metric.position].split(metric.separator)[1])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self.test_data

        self._read_fpm_config()

        listen = self.fpm_config.get(self.pool, "listen")
        status_path = self.fpm_config.get(self.pool, "pm.status_path")
        if listen.startswith("/"):
            connector = FcgiConnector(socket_file=listen, uri=status_path)
        else:
            if ":" in listen:
                host, port = listen.split(":")
                connector = FcgiConnector(host=host, port=port, uri=status_path)
            else:
                connector = FcgiConnector(port=listen, host="127.0.0.1", uri=status_path)

        code, headers, data, error = connector.get()

        if not code.startswith("200"):
            self.logger.error("status: got response, but not correct")
            return None

        Cache.write(self.name, data)

        return data.split("\n")

    def _read_fpm_config(self):
        if self.fpm_config is not None:
            return

        fpm_config_path = self.config.get("fpm_config_path", "/etc/php-fpm.d")

        self.fpm_config = ConfigParser()
        self.fpm_config.read(utils.find_files_by_extension(fpm_config_path, "conf"))

        if len(self.fpm_config.sections()) == 0:
            self.logger.error("Can't read from PHP-FPM config")

        if self.pool not in self.fpm_config.sections():
            raise CheckFail("Pool %s does not exist in config files" % self.pool)

        if not self.fpm_config.has_option(self.pool, "pm.status_path"):
            raise CheckFail("Status path is not configured for pool %s" % self.pool)



from check import Check, MetricType, Metric
from lib.mysqlconnector import MySQLConnector
from lib.cache import Cache


class Sphinx(Check):
    name = "sphinx"

    def _init_metrics(self):
        self.metrics = {
            "uptime": Metric("uptime", MetricType.Integer, 0),
            "connections": Metric("connections", MetricType.Integer, 1),
            "maxed_out": Metric("maxed_out", MetricType.Integer, 2),
            "command_search": Metric("command_search", MetricType.Integer, 3),
            "command_excerpt": Metric("command_excerpt", MetricType.Integer, 4),
            "command_update": Metric("command_update", MetricType.Integer, 5),
            "command_keywords": Metric("command_keywords", MetricType.Integer, 6),
            "command_persist": Metric("command_persist", MetricType.Integer, 7),
            "command_status": Metric("command_status", MetricType.Integer, 8),
            "command_flushattrs": Metric("command_flushattrs", MetricType.Integer, 9),
            "agent_connect": Metric("agent_connect", MetricType.Integer, 10),
            "agent_retry": Metric("agent_retry", MetricType.Integer, 11),
            "queries": Metric("queries", MetricType.Integer, 12),
            "dist_queries": Metric("dist_queries", MetricType.Integer, 13),
            "query_wall": Metric("query_wall", MetricType.Float, 14),
            "query_cpu": Metric("query_cpu", MetricType.Float, 15),
            "dist_wall": Metric("dist_wall", MetricType.Float, 16),
            "dist_local": Metric("dist_local", MetricType.Float, 17),
            "dist_wait": Metric("dist_wait", MetricType.Float, 18),
            "query_reads": Metric("query_reads", MetricType.Float, 19),
            "query_readkb": Metric("query_readkb", MetricType.Float, 20),
            "query_readtime": Metric("query_readtime", MetricType.Float, 21),
            "avg_query_wall": Metric("avg_query_wall", MetricType.Float, 22),
            "avg_query_cpu": Metric("avg_query_cpu", MetricType.Float, 23),
            "avg_dist_wall": Metric("avg_dist_wall", MetricType.Float, 24),
            "avg_dist_local": Metric("avg_dist_local", MetricType.Float, 25),
            "avg_dist_wait": Metric("avg_dist_wait", MetricType.Float, 26),
            "avg_query_reads": Metric("avg_query_reads", MetricType.Float, 27),
            "avg_query_readkb": Metric("avg_query_readkb", MetricType.Float, 28),
            "avg_query_readtime": Metric("avg_query_readtime", MetricType.Float, 29),
        }

    def _get(self, metric=None, *args, **kwargs):
        self.test_data = self._load_data()
        return self._get_value(self.metrics[metric])

    def _get_value(self, metric):
        if self.test_data[metric.key] == "OFF" and not metric.type == MetricType.String:
            self.test_data[metric.key] = ""

        return self._correct_type(metric.type, self.test_data[metric.key])

    def _load_data(self):
        self.test_data = Cache.read(self.name)
        if self.test_data is not None:
            return self._format_data(self.test_data)

        connector = MySQLConnector(host=self.config.get("host", "127.0.0.1"), port=self.config.get("port", 9306))
        data = connector.get("SHOW STATUS")
        Cache.write(self.name, data)

        return self._format_data(data)

    def _format_data(self, data):
        tmp = {}
        for value in data:
            tmp[value["Counter"]] = value["Value"]

        return tmp

    def _filter_data(self, linenumber, separator, key):
        data = self.test_data[linenumber].split(separator)[1].count(key)
        self.test_data[linenumber] = str(key) + ":" + str(data)

